@@ -5,16 +5,14 @@
 
     Builder superclass for all builders.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 import gettext
 import io
 import os
 import re
-import warnings
 from os import path
-from time import gmtime
 from datetime import datetime
 from collections import namedtuple
 
@@ -23,9 +21,15 @@ from babel.messages.pofile import read_po
 from babel.messages.mofile import write_mo
 
 from sphinx.errors import SphinxError
-from sphinx.util.osutil import walk
-from sphinx.util import SEP
+from sphinx.util import logging
+from sphinx.util.osutil import SEP, walk
 
+logger = logging.getLogger(__name__)
+
+if False:
+    # For type annotation
+    from typing import Callable, List, Set  # NOQA
+    from sphinx.environment import BuildEnvironment  # NOQA
 
 LocaleFileInfoBase = namedtuple('CatalogInfo', 'base_dir,domain,charset')
 
@@ -34,32 +38,48 @@ class CatalogInfo(LocaleFileInfoBase):
 
     @property
     def po_file(self):
+        # type: () -> unicode
         return self.domain + '.po'
 
     @property
     def mo_file(self):
+        # type: () -> unicode
         return self.domain + '.mo'
 
     @property
     def po_path(self):
+        # type: () -> unicode
         return path.join(self.base_dir, self.po_file)
 
     @property
     def mo_path(self):
+        # type: () -> unicode
         return path.join(self.base_dir, self.mo_file)
 
     def is_outdated(self):
+        # type: () -> bool
         return (
             not path.exists(self.mo_path) or
             path.getmtime(self.mo_path) < path.getmtime(self.po_path))
 
     def write_mo(self, locale):
-        with io.open(self.po_path, 'rt', encoding=self.charset) as po:
-            with io.open(self.mo_path, 'wb') as mo:
-                write_mo(mo, read_po(po, locale))
+        # type: (unicode) -> None
+        with io.open(self.po_path, 'rt', encoding=self.charset) as file_po:
+            try:
+                po = read_po(file_po, locale)
+            except Exception:
+                logger.warning('reading error: %s', self.po_path)
+                return
+
+        with io.open(self.mo_path, 'wb') as file_mo:
+            try:
+                write_mo(file_mo, po)
+            except Exception:
+                logger.warning('writing error: %s', self.mo_path)
 
 
 def find_catalog(docname, compaction):
+    # type: (unicode, bool) -> unicode
     if compaction:
         ret = docname.split(SEP, 1)[0]
     else:
@@ -69,18 +89,20 @@ def find_catalog(docname, compaction):
 
 
 def find_catalog_files(docname, srcdir, locale_dirs, lang, compaction):
+    # type: (unicode, unicode, List[unicode], unicode, bool) -> List[unicode]
     if not(lang and locale_dirs):
         return []
 
     domain = find_catalog(docname, compaction)
-    files = [gettext.find(domain, path.join(srcdir, dir_), [lang])
+    files = [gettext.find(domain, path.join(srcdir, dir_), [lang])  # type: ignore
              for dir_ in locale_dirs]
-    files = [path.relpath(f, srcdir) for f in files if f]
-    return files
+    files = [path.relpath(f, srcdir) for f in files if f]  # type: ignore
+    return files  # type: ignore
 
 
 def find_catalog_source_files(locale_dirs, locale, domains=None, gettext_compact=False,
                               charset='utf-8', force_all=False):
+    # type: (List[unicode], unicode, List[unicode], bool, unicode, bool) -> Set[CatalogInfo]
     """
     :param list locale_dirs:
        list of path as `['locale_dir1', 'locale_dir2', ...]` to find
@@ -97,10 +119,11 @@ def find_catalog_source_files(locale_dirs, locale, domains=None, gettext_compact
        default is False.
     :return: [CatalogInfo(), ...]
     """
+    catalogs = set()  # type: Set[CatalogInfo]
+
     if not locale:
-        return []  # locale is not specified
+        return catalogs  # locale is not specified
 
-    catalogs = set()
     for locale_dir in locale_dirs:
         if not locale_dir:
             continue  # skip system locale directory
@@ -126,6 +149,7 @@ def find_catalog_source_files(locale_dirs, locale, domains=None, gettext_compact
 
     return catalogs
 
+
 # date_format mappings: ustrftime() to bable.dates.format_datetime()
 date_format_mappings = {
     '%a': 'EEE',     # Weekday as locale’s abbreviated name.
@@ -157,7 +181,8 @@ date_format_mappings = {
 }
 
 
-def babel_format_date(date, format, locale, warn=None, formatter=babel.dates.format_date):
+def babel_format_date(date, format, locale, formatter=babel.dates.format_date):
+    # type: (datetime, unicode, unicode, Callable) -> unicode
     if locale is None:
         locale = 'en'
 
@@ -172,73 +197,68 @@ def babel_format_date(date, format, locale, warn=None, formatter=babel.dates.for
         # fallback to English
         return formatter(date, format, locale='en')
     except AttributeError:
-        if warn:
-            warn('Invalid date format. Quote the string by single quote '
-                 'if you want to output it directly: %s' % format)
-
+        logger.warning('Invalid date format. Quote the string by single quote '
+                       'if you want to output it directly: %s', format)
         return format
 
 
-def format_date(format, date=None, language=None, warn=None):
-    if format is None:
-        format = 'medium'
-
+def format_date(format, date=None, language=None):
+    # type: (str, datetime, unicode) -> unicode
     if date is None:
         # If time is not specified, try to use $SOURCE_DATE_EPOCH variable
         # See https://wiki.debian.org/ReproducibleBuilds/TimestampsProposal
         source_date_epoch = os.getenv('SOURCE_DATE_EPOCH')
         if source_date_epoch is not None:
-            date = gmtime(float(source_date_epoch))
+            date = datetime.utcfromtimestamp(float(source_date_epoch))
         else:
             date = datetime.now()
 
-    if re.match('EEE|MMM|dd|DDD|MM|WW|medium|YY', format):
-        # consider the format as babel's
-        warnings.warn('LDML format support will be dropped at Sphinx-1.5',
-                      DeprecationWarning)
+    result = []
+    tokens = re.split('(%.)', format)
+    for token in tokens:
+        if token in date_format_mappings:
+            babel_format = date_format_mappings.get(token, '')
 
-        return babel_format_date(date, format, locale=language, warn=warn,
-                                 formatter=babel.dates.format_datetime)
-    else:
-        # consider the format as ustrftime's and try to convert it to babel's
-        result = []
-        tokens = re.split('(%.)', format)
-        for token in tokens:
-            if token in date_format_mappings:
-                babel_format = date_format_mappings.get(token, '')
-
-                # Check if we have to use a different babel formatter then
-                # format_datetime, because we only want to format a date
-                # or a time.
-                if token == '%x':
-                    function = babel.dates.format_date
-                elif token == '%X':
-                    function = babel.dates.format_time
-                else:
-                    function = babel.dates.format_datetime
-
-                result.append(babel_format_date(date, babel_format, locale=language,
-                                                formatter=function))
+            # Check if we have to use a different babel formatter then
+            # format_datetime, because we only want to format a date
+            # or a time.
+            if token == '%x':
+                function = babel.dates.format_date
+            elif token == '%X':
+                function = babel.dates.format_time
             else:
-                result.append(token)
+                function = babel.dates.format_datetime
 
-        return "".join(result)
+            result.append(babel_format_date(date, babel_format, locale=language,
+                                            formatter=function))
+        else:
+            result.append(token)
+
+    return "".join(result)
 
 
 def get_image_filename_for_language(filename, env):
+    # type: (unicode, BuildEnvironment) -> unicode
     if not env.config.language:
         return filename
 
     filename_format = env.config.figure_language_filename
-    root, ext = path.splitext(filename)
+    d = dict()
+    d['root'], d['ext'] = path.splitext(filename)
+    dirname = path.dirname(d['root'])
+    if dirname and not dirname.endswith(path.sep):
+        dirname += path.sep
+    d['path'] = dirname
+    d['basename'] = path.basename(d['root'])
+    d['language'] = env.config.language
     try:
-        return filename_format.format(root=root, ext=ext,
-                                      language=env.config.language)
+        return filename_format.format(**d)
     except KeyError as exc:
         raise SphinxError('Invalid figure_language_filename: %r' % exc)
 
 
 def search_image_for_language(filename, env):
+    # type: (unicode, BuildEnvironment) -> unicode
     if not env.config.language:
         return filename
 
